import zhinst.ziPython as ziPython 
#from . import ziPython
import re 

def autoDetect(daq, exclude = []):
    """
    Returns a string with the first device that is not
    in the exclude list. Will throw a RuntimeError if no
    device was found.
    """
    if not isinstance(daq,ziPython.ziDAQServer):
        raise RuntimeError("First argument must be an instance of ziPython.ziDAQServer");
    nodes = daq.listNodes('/', 0);
    devices = filter(lambda node: re.match("dev*", node, re.IGNORECASE), nodes);
    if not isinstance(exclude,list):
        exclude = [exclude];
    exclude = list(x.lower() for x in exclude);
    devices = filter(lambda node: node.lower() not in exclude, devices);
    if len(devices) == 0 :               
        raise RuntimeError("No DUT found. Make sure that the USB cable (HF2) or ethernet cable (UHF)is connected to the host and the device is turned on.");
    # Found at least one device -> selection valid.
    # Select the first one
    device = devices[0].lower();
    print("autoDetect selected the device %s for the measurement." % device);
    return device;

def devices(daq):
    """
    Returns a list of devices.
    """
    if not isinstance(daq,ziPython.ziDAQServer):
        raise RuntimeError("First argument must be an instance of ziPython.ziDAQServer");
    nodes = daq.listNodes('/', 0);
    devices = filter(lambda node: re.match("dev*", node, re.IGNORECASE), nodes);
    devices = list(x.lower() for x in list(devices));
    return devices;

def autoConnect(defaultPort=None, APILevel=None):
    """
    Try to connect to a Zurich Instruments server with an available
    device. 

    defaultPort should be either None, 8004 (UHF) or 8005 (HF2).

    For HF2, APILevel can only be 1. For UHF it can be either 1 or 4 (latest).

    If defaultPort is not specified (=None) first try to connect to a UHF, if
    no server devices are found then try to connect to an HF2. This behaviour
    is useful for the API examples. If we cannot connect to a server and/or
    detect a connected device raise a RunTimeError.

    If defaultPort is 8004 try to connect to a UHF; if it is 8005 try to
    connect to an HF2. If no server and device is detected on this port raise
    a RunTimeError.
    """
    if defaultPort is None:
        defaultPort = 8005
        secondaryPort = 8004
    elif defaultPort in [8004,8005]:
        # If a port is specified, then don't try to connect to a secondary port
        secondaryPort = None
    else:
        raise RuntimeError("autoConnect(): input argument defaultPort (%d) must be either 8004 or 8005." % defaultPort);
    if APILevel is None:
        # Note: level 1 used by default for both UHF and HF2, otherwise
        # backwards compatibility not maintained.
        APILevel = 1

    try:
        daq = ziPython.ziDAQServer('localhost', defaultPort, APILevel);
        devs = devices(daq);
    except RuntimeError:
        devs = [];
    if len(devs) > 0:
        # We have a server running and a device, we're done
        print "autoConnect connected to a server on port %d using API level %d." % (defaultPort, APILevel)
        return daq

    # secondaryPort is None if defaultPort is specified as an input argument, no success raise an error.
    if secondaryPort is None:
        raise RuntimeError("autoConnect(): failed to find a running server or failed to find a device connected to the server on port %d. Please ensure that the correct Zurich Instruments server is running for your device and that your device is connected to the server (try connecting first via the User Interface)." % defaultPort);

    try:
        daq = ziPython.ziDAQServer('localhost', secondaryPort, APILevel);
        devs = devices(daq);
    except RuntimeError:
        devs = [];
    if len(devs) > 0:
        # We have a server running and a device, we're done
        print "autoConnect connected to a server on port %d using API level %d." % (secondaryPort, APILevel)
        return daq
    # Error checking, keep it simple
    raise RuntimeError("autoConnect(): failed to find a running server or failed to find a device connected to the server. Please ensure that the correct Zurich Instruments server is running for your device and that your device is connected to the server (try connecting first via the User Interface).");
